-- GLOBAL --
UCM = select(2, ...); 
-- MODULE --
local _G = getfenv(0);
setmetatable(UCM, {__index = _G});
setfenv(1, UCM);
-- LOCAL --
title		= "UnlimitedChatMessage"
version		= "1.1.0"
titleFull	= title.." "..version

coreFrame = CreateFrame("Frame");
core	= LibStub("AceAddon-3.0"):NewAddon(coreFrame, title, "AceConsole-3.0", "AceHook-3.0") -- 
local L = LibStub("AceLocale-3.0"):GetLocale(title, true)

local P; --db.profile

defaultMaxLetters = 255;--blizzard's default limit. don't touch!
defaultMaxBytes = 256;

queuedOutgoingMsgs = {};

--Default Settings
db				= {};
defaultSettings	= {profile = {}}
defaultSettings.profile.confirmLongMsgs = true

-- Ace3 Functions
function core:OnInitialize()
	db = LibStub("AceDB-3.0"):New("UCM_DB", defaultSettings)
	CoreOptionsTable.args.profiles = LibStub("AceDBOptions-3.0"):GetOptionsTable(db)--save option profile or load another chars opts.
	db.RegisterCallback(self, "OnProfileChanged", "OnProfileChanged")
	db.RegisterCallback(self, "OnProfileCopied", "OnProfileChanged")
	db.RegisterCallback(self, "OnProfileReset", "OnProfileChanged")
	db.RegisterCallback(self, "OnProfileDeleted", "OnProfileChanged")

	P = db.profile;

	core:RegisterChatCommand("ucm", "MySlashProcessorFunc")

	local config = LibStub("AceConfig-3.0");
	local dialog = LibStub("AceConfigDialog-3.0");
	config:RegisterOptionsTable(title, CoreOptionsTable)
	coreOpts = dialog:AddToBlizOptions(title, "UCM "..version)
end


----------------------------------------------
function core:MySlashProcessorFunc(input)	--
-- /da function brings up the UI options.	--
----------------------------------------------
	InterfaceOptionsFrame_OpenToCategory(coreOpts)
end

function core:OnEnable()
	core:RawHook("SendChatMessage", true)
	
	ChatFrameEditBox:SetMaxLetters(0)--0=unlimited
	ChatFrameEditBox:SetMaxBytes(0)--0=unlimited
end

----------------------------------------------------------------------
function core:OnProfileChanged(...)									--
-- User has reset proflie, so we reset our spell exists options.	--
----------------------------------------------------------------------
	-- Shut down anything left from previous settings
	self:Disable()
	-- Enable again with the new settings
	self:Enable()
end


function core:OnDisable()
	--return chatbox maxs to defaults.
	ChatFrameEditBox:SetMaxLetters(defaultMaxLetters)
	ChatFrameEditBox:SetMaxBytes(defaultMaxBytes)
end

------------------------------------------------------------------
function core:SendChatMessage(...)								--
-- Our hook to SendChatMessage. If msg is too long we split it.	--
------------------------------------------------------------------
	local msg, chatType ,language ,channel = ...;
	if msg:len() > defaultMaxLetters then
		local chunks = SplitIntoChunks(msg)

		--P.confirmLongMsgs
		if table.getn(chunks) > 5 and P.confirmLongMsgs == true then
			for i=1, table.getn(chunks) do 
				table.insert(queuedOutgoingMsgs, {msg=chunks[i], chatType=chatType, language=language, channel=channel})
			end
			StaticPopup_Show("UCM_CONFIRMMSG");
		else
			for i=1, table.getn(chunks) do 
				_G.ChatThrottleLib:SendChatMessage("BULK", "UCM", chunks[i], chatType, nil, channel);
			end
		end
		return;
	end

	core.hooks.SendChatMessage(...)
end


function paddString(str, paddingChar, minLength, paddRight)
    str = tostring(str or "");
    paddingChar = tostring(paddingChar or " ");
    minLength = tonumber(minLength or 0);
    while(string.len(str) < minLength) do
        if(paddRight) then
            str = str..paddingChar;
        else
            str = paddingChar..str;
        end
    end
    return str;
end

function SplitIntoChunks(longMsg)--	/script UCM.SplitIntoChunks("")
	local splitMessageLinks = {};
	
	--Replace links with long strings of zeroes.
	longMsg, results = string.gsub(longMsg, "(|H[^|]+|h[^|]+|h)", function(theLink)
			table.insert(splitMessageLinks, theLink);
			return "\001\002"..paddString(#splitMessageLinks, "0", string.len(theLink)-4).."\003\004";
	end);
	

	local words = {}
	for v in string.gmatch(longMsg, "[^ ]+") do
	
		--Check if 'word' is longer then 254 characters. (wtf?) anyway split long string at the 254 char mark.
		if v:len() > defaultMaxLetters then
			local shortPart, remainingPart = nil, v;
			local i=1;
			while remainingPart and remainingPart:len() > 0 do
				shortPart, remainingPart = GetShorterString(remainingPart);
				if shortPart and shortPart ~= "" then
					table.insert(words, shortPart)
				end
				
				if i>10 then break; end
				i=i+1;
			end
		else
			table.insert(words, v)
		end
	end

	local temp = "";
	local chunks = {}
	for i=1, table.getn(words) do 
		if temp:len() + words[i]:len() <= (defaultMaxLetters - 1) then
			temp = temp.." "..words[i];
		else
			temp = string.gsub(temp, "\001\002%d+\003\004", function(link)
				local index = _G.tonumber(string.match(link, "(%d+)"));
				return splitMessageLinks[index] or link;
			end);
			table.insert(chunks, temp);
			temp = words[i];
		end
	end

	if temp:len() > 0 then
		temp = string.gsub(temp, "\001\002%d+\003\004", function(link)
			local index = _G.tonumber(string.match(link, "(%d+)"));
			return splitMessageLinks[index] or link;
		end);
		table.insert(chunks, temp);
	end

	return chunks;
end

------------------------------------------------------------------------------------------
function GetShorterString(longMsg)														--
-- Used to split msgs from their spaces, now only used to split msgs with no spaces. =/	--
------------------------------------------------------------------------------------------
--~ 	local max = defaultMaxLetters;
--~ 	if max > longMsg:len() then
--~ 		max = longMsg:len()
--~ 	end
--~ 	
--~ 	if string.find(string.sub(longMsg, 1, max), " ") then
--~ 		for i=max, 1, -1 do 
--~ 			if string.sub(longMsg, i, i) == " " then
--~ 				local shortPart = string.sub(longMsg, 1, i-1);
--~ 				local remainingPart = string.sub(longMsg, i+1, longMsg:len());
--~ 				return shortPart, remainingPart;
--~ 			end
--~ 		end
--~ 	end
	
	local shortText = string.sub(longMsg, 1, defaultMaxLetters-1)
	local remainingPart = string.sub(longMsg, defaultMaxLetters, longMsg:len())
	return shortText, remainingPart;
end


_G.StaticPopupDialogs["UCM_CONFIRMMSG"] = {
	text = "Send long msg?",
	button1 = YES,
	button2 = NO,
	OnAccept = function(self)
		for i=1, table.getn(queuedOutgoingMsgs) do 
			_G.ChatThrottleLib:SendChatMessage("BULK", "UCM", queuedOutgoingMsgs[i].msg, queuedOutgoingMsgs[i].chatType, nil, queuedOutgoingMsgs[i].channel);
		end
	end,
	
	OnHide = function(self)
		queuedOutgoingMsgs = {};
	end,
	OnShow = function(self)
		self.text:SetText(L["Split outgoing message into %d parts?"]:format(table.getn(queuedOutgoingMsgs)))
	end,
	timeout = 0,
	hideOnEscape = 1,
};

-- UI
CoreOptionsTable = {
	name = title,
	type = "group",
	childGroups = "tab",

	args = {
		core={
			name = L["Core"],
			type = "group",
			order = 1,
			args={}
		},
	},
}

CoreOptionsTable.args.core.args.enable = {
	type = "toggle",	order	= 1,
	name	= L["Enable"],
	desc	= L["Enables / Disables the addon"],
	set = function(info,val) 
		if val == true then
			core:Enable();
		else
			core:Disable();
		end
	end,
	get = function(info) return core:IsEnabled() end
}

CoreOptionsTable.args.core.args.confirmLongMsgs = {
	type = "toggle",	order	= 2,
	
	name = L["Confirm Long Messages"],	
	desc = L["Show a confirm window if splitting message into 5+ parts."],
	set = function(info,val) 
		P.confirmLongMsgs = val;
	end,
	get = function(info) return P.confirmLongMsgs end
}